@@ -1,7 +1,7 @@
 import attr
 
 
-@attr.s(auto_attribs=True, kw_only=True, frozen=True)
+@attr.s(auto_attribs=True, kw_only=True)
 class COM:
     """
     abstract base class for all components
